import inspect


def filter_dict(dict_to_filter, keywords, default=None):
    new_dict = {key: dict_to_filter.get(key, default) for key in keywords}
    return new_dict


def get_method_keywords(method, return_contains_args_kwargs=True):
    sig = inspect.signature(method)
    filter_keys = {
        param.name:param.default if param.default is not inspect._empty else None
        for param in sig.parameters.values()
        if param.kind == param.POSITIONAL_OR_KEYWORD or param.kind == param.KEYWORD_ONLY
    }
    if not return_contains_args_kwargs:
        return filter_keys
    return (
        filter_keys,
        len(
            [
                param.name
                for param in sig.parameters.values()
                if param.kind == param.VAR_POSITIONAL
            ]
        )
        > 0,
        len(
            [
                param.name
                for param in sig.parameters.values()
                if param.kind == param.VAR_KEYWORD
            ]
        )
        > 0,
    )


def call_method(target, args=None, kwargs=None):
    if kwargs is None:
        kwargs = {}
    if args is None:
        args = []

    target_keywords, with_args, with_kwargs = get_method_keywords(target)
    if with_kwargs:
        target_dict = dict.fromkeys(target_keywords, None)
        target_dict.update(kwargs)
    else:
        target_dict = filter_dict(dict_to_filter=kwargs, keywords=target_keywords)
    return target(**target_dict)


if __name__ == "__main__":

    def f(a, b=None, **kwargs):
        print(a, b, kwargs)

    kwargs = {"a": 0, "b": 1, "c": 2}

    target_keywords, with_args, with_kwargs = get_method_keywords(f)

    print(target_keywords, with_args, with_kwargs)
    print(filter_dict(dict_to_filter=kwargs, keywords=target_keywords))
    call_method(f, kwargs=kwargs)



import setuptools

setup = dict(
    name="FilterDict",
    version="0.2",
    author="Julian Kimmig",
    author_email="julian-kimmig@gmx.net",
    description="Store data in JSON format",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/JulianKimmig/filter_dict",
    py_modules=["filter_dict"],
    entry_points="""
             [console_scripts]
             filter_dict=filter_dict:filter_dict""",
    # packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
if __name__ == "__main__":
    setuptools.setup(**setup)



import os
import shutil
import subprocess
import sys
import time


def create_n_upload():
    os.chdir(os.path.abspath(os.path.dirname(__file__)))
    some_command = (
        sys.executable + " " + os.path.basename(__file__) + " sdist bdist_wheel"
    )
    p = subprocess.Popen(some_command, stdout=subprocess.PIPE)
    while True:
        output = p.stdout.readline().decode("utf-8")
        if output == "" and p.poll() is not None:
            break
        if output:
            print(output.strip())
        time.sleep(0.01)
    rc = p.poll()
    some_command = sys.executable + " -m twine upload dist/*"
    p = subprocess.Popen(some_command, stdout=subprocess.PIPE)
    while True:
        output = p.stdout.readline().decode("utf-8")
        if output == "" and p.poll() is not None:
            break
        if output:
            print(output.strip())
        time.sleep(0.01)
    rc = p.poll()


if __name__ == "__main__":
    shutil.rmtree(
        os.path.join(os.path.dirname(os.path.abspath(__file__)), "dist"),
        ignore_errors=True,
    )
    shutil.rmtree(
        os.path.join(os.path.dirname(os.path.abspath(__file__)), "build"),
        ignore_errors=True,
    )
    if len(sys.argv) == 1:
        create_n_upload()
    else:
        import setuptools
        from setup import setup

        setup = setup
        setup["version"] = setup["version"] + "." + str(int(time.time()))
        print(setup["version"])
        setuptools.setup(**setup)
